<?php

namespace ONLINE_ADA;

use FoxORM\GibberishDetector\Gibberish;

class Plugin
{

	/**
	 * Info about the plugin e.g. Version number and File path
	 *
	 * @var array
	 */
	public $info;

	/**
	 * Plugin instance for use throughout plugin
	 *
	 * @var Plugin
	 */
	public static $instance = null;

	public static function instance($info = [])
	{
		if (is_null(self::$instance)) {
			self::$instance = new self($info);
		}

		return self::$instance;
	}

	private function __construct($info)
	{
		$this->info = $info;

		if (!file_exists($this->info["path"] . "vendor/autoload.php")) return;

		require_once $this->info["path"] . "vendor/autoload.php";

		register_activation_hook($this->info['file'], [$this, 'activate']);
		register_deactivation_hook($this->info['file'], [$this, 'deactivate']);
	}

	// private function setupGib(){
	// 	$path_to_training_files = $this->info["path"] .  "vendor/foxorm/gibberish-detector/sample/phrases/";
		
	// 	$trainingSuccessful = Gibberish::train(
	// 		$path_to_training_files . "big.txt",
	// 		$path_to_training_files . "good.txt", 
	// 		$path_to_training_files . "bad.txt", 
	// 		$this->info["gibberish_matrix"]
	// 	);
	// }

	private function createUpdatesArray(){
		if(!get_option("oadaas_updates_array")){
			add_option("oadaas_updates_array", [
				"3.0.0" => [
					"scan_cutoff_date" => time()
				]
			]);
		}
	}

	public function activate($path)
	{
		add_option('oadaas-show-activation-message', 1, '', 'yes');
		Helper::create_table();
		if (!file_exists(wp_upload_dir()["basedir"] . "/oadaas")) {
			mkdir(wp_upload_dir()["basedir"] . "/oadaas", 0777, true);
		}
		flush_rewrite_rules();
		self::create_sitemap_initial();
		$this->createUpdatesArray();
		// $this->setupGib();
	}

	private static function create_sitemap_initial()
	{
        //Logic for scanning database for all pages here
		if(!get_option("_oadaas_sitemap")){
			$sitemap = [];
			$sitemap["non_query_links"][] = sanitize_text_field(untrailingslashit(home_url()));

			$result = update_option("_oadaas_sitemap", $sitemap);
			if (!$result) {
				error_log(self::$instance->info["name_pretty"] . ': failed to create sitemap in database');
			}
		}
	}

	public function deactivate()
	{
		$result = delete_option("_oadaas_sitemap");
		if (!$result) {
			error_log(self::$instance->info["name_pretty"] .": Error deleting sitemap from database");
		}
		flush_rewrite_rules();
	}
}