<?php
namespace ONLINE_ADA;

use ONLINE_ADA\Helper;
use ONLINE_ADA\Plugin;
?>


<div class="wrap">
	<h1><?php echo get_admin_page_title(); ?></h1>
	<div class="wcag-main">
		<div class="wcag-sidebar">
			<div class="wcag-widget wcag-limelight">
				<p><strong><abbr title="Americans with Disabilities Act">Online ADA</abbr> Accessibility Tool</strong></p>
				<p>
					<a href="<?php echo esc_attr(admin_url('post-new.php?post_type=wcag_scan')); ?>" class="button button-primary">New Scan</a>
					<a href="<?php echo esc_attr(admin_url('edit.php?post_type=wcag_scan')); ?>" class="button button-secondary">Scan Results</a>
				</p>
			</div>
		</div>

		<div class="wcag-content">

			<p>Below is the list of WCAG 2.1 requirements for digital accessibility. This list covers level A and AA requirements. AAA, the highest rating, is not included because it's not a legal requirement yet. Included below are the exact WCAG requirements, notes on how the scan will help, and helpful tips.</p>
		    <p>Click Here to see the complete WCAG 2.1 checklist and guide: <a target="_blank" title="WCAG 2.1 quick reference guide" rel="external" href="https://www.w3.org/WAI/WCAG20/quickref/">WCAG 2.1 Quick Reference Guide</a> on the <a href="https://www.w3.org/WAI" target="_blank" rel="external">Web Accessibility Initiative (WAI)</a> homepage</p>

			<dl class="wcag-list">
                <?php foreach (Helper::getWcagArticlesModel() AS $article) : ?>
                    <div class="principle-heading"><?php echo $article["principle_heading"]; ?></div>
                    <div class="principle-subheading"><?php echo $article["principle_sub_heading"]; ?></div>
                        <?php foreach ($article["principle_guidelines"] AS $guideline) { ?>
                            <div class="guideline-wrap">
                                <div class="guideline-heading"><?php echo $guideline["heading"]; ?></div>
                                <div class="guideline-sub-heading"><?php echo $guideline["subheading"]; ?></div>
                                <?php foreach ($guideline["level"] AS $level => $value) { ?>
                                    <?php if (!empty($value)) : ?>
                                        <?php foreach ($value AS $index => $data) { ?>
                                            <dl>
                                                <dt id="<?php echo $index; ?>" class="checklist-article-number">
													<a href="<?php echo $data['url']; ?>" target="_blank" rel="external" ><?php echo $index .
													" " . $data['title'] . " <span class='checklist-level'>Level " . $level . "</span>"; ?></a>
                                                </dt>
                                                <dd>
                                                    <blockquote class="checklist-title">
                                                        <div class="quotes" ><?php echo $data['message']; ?></div>
                                                    </blockquote>
                                                    <?php if (!empty($data["tip"])) : ?>
                                                        <div class="wcag-wp-desc">
                                                            <?php echo "Tip: " . $data['tip']; ?>
                                                        </div>
                                                    <?php endif ?>
                                                </dd>
                                            </dl>
                                        <?php
										} ?>
                                    <?php endif ?>
                                <?php
								} ?>
                            </div>
                        <?php
					} ?>

                <?php endforeach ?>

            </dl>

			<div class="wcag-footnotes">
				<h3>Disclaimer:</h3>

				<ul class="ul-disc">
					<li><strong>Third Party Plugins:</strong> Some plugins, especially those which do not follow WordPress standards or avoid WordPress API's, are not be accounted for by the accessibility scanning plugin. </li>

					<li><strong>"Hard-Coded" Markup:</strong> Plugins and themes that included "hard-coded" markup will not be accounted for during scans. This includes files that have markup that is directly included by template files, and are not editable within the dashboard.</li>
				</ul>

			</div>

		</div>

	</div>

</div>